package github.benjamin.equipreservebackend.controller;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import github.benjamin.equipreservebackend.entity.Device;
import github.benjamin.equipreservebackend.response.ResponseResult;
import github.benjamin.equipreservebackend.service.DeviceService;
import github.benjamin.equipreservebackend.service.ReservationService;
import github.benjamin.equipreservebackend.vo.DeviceAdminVO;
import github.benjamin.equipreservebackend.vo.DeviceStatsVO;
import github.benjamin.equipreservebackend.vo.DeviceUserVO;
import github.benjamin.equipreservebackend.vo.TimeRangeVO;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/device")
public class DeviceController {

    private final DeviceService deviceService;

    private final ReservationService reservationService;

    @PreAuthorize("hasRole('USER')")
    @GetMapping
    public ResponseResult<Page<DeviceUserVO>> getUserDevices(@RequestParam(defaultValue = "1") Integer page,
                                                             @RequestParam(defaultValue = "10") Integer size,
                                                             @RequestParam(required = false) String name) {
        Page<Device> pageRequest = new Page<>(page, size);
        Page<DeviceUserVO> res = deviceService.getUserDevices(pageRequest, name);
        return ResponseResult.success(res);
    }

    @PreAuthorize("hasRole('USER')")
    @GetMapping("/unavailable-times/{id}")
    public ResponseResult<List<TimeRangeVO>> getUnavailableTimes(@PathVariable Long id) {
        List<TimeRangeVO> res = reservationService.getUnavailableTimes(id);
        return ResponseResult.success(res);
    }

    @PreAuthorize("hasRole('DEVICE_ADMIN')")
    @PostMapping("/{userId}")
    public ResponseResult<Device> addDevice(@PathVariable("userId") Long userId,
                                            @RequestBody Device device) {
        deviceService.addDevice(userId, device);
        return ResponseResult.success(device);
    }

    @PreAuthorize("hasRole('DEVICE_ADMIN')")
    @DeleteMapping("/{id}")
    public ResponseResult<?> deleteDevice(@PathVariable("id") Long id) {
        deviceService.deleteDevice(id);
        return ResponseResult.success();
    }

    @PreAuthorize("hasRole('DEVICE_ADMIN')")
    @PutMapping("/{id}")
    public ResponseResult<?> updateDevice(@PathVariable("id") Long id,
                                          @RequestBody Device device) {
        device.setId(id);
        Device updatedDevice = deviceService.updateDevice(device);
        return ResponseResult.success(updatedDevice);
    }

    @PreAuthorize("hasRole('DEVICE_ADMIN')")
    @GetMapping("/{userId}")
    public ResponseResult<Page<DeviceAdminVO>> getAdminDevice(@PathVariable("userId") Long userId,
                                                              @RequestParam(defaultValue = "1") Integer page,
                                                              @RequestParam(defaultValue = "10") Integer size,
                                                              @RequestParam(required = false) String name) {

        Page<Device> pageRequest = new Page<>(page, size);
        return ResponseResult.success(deviceService.getAdminDevice(pageRequest, userId, name));
    }

    @PreAuthorize("hasRole('DEVICE_ADMIN')")
    @PostMapping("/{id}/image")
    public ResponseResult<?> uploadImage(@PathVariable("id") Long id,
                                         @RequestParam("image") MultipartFile image) throws IOException {
        String imagePath = deviceService.saveImage(id, image);
        return ResponseResult.success(imagePath);
    }

    @PreAuthorize("hasRole('ADMIN')")
    @GetMapping("/usage-stats")
    public ResponseResult<List<DeviceStatsVO>> getDeviceUsageStats(@RequestParam @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate start,
                                                                   @RequestParam @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate end) {
        return ResponseResult.success(deviceService.getDeviceUsageStats(start, end));
    }

    @PreAuthorize("hasRole('ADMIN')")
    @GetMapping("/usage-stats/export")
    public void exportDeviceUsageStats(@RequestParam @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate start,
                                       @RequestParam @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate end,
                                       HttpServletResponse response) throws IOException {
        String fileName = String.format("设备使用统计_%s-%s.xlsx", start.format(DateTimeFormatter.ISO_DATE), end.format(DateTimeFormatter.ISO_DATE));
        String encodedName = URLEncoder.encode(fileName, StandardCharsets.UTF_8);
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + encodedName);

        // 查询数据
        List<DeviceStatsVO> data = deviceService.getDeviceUsageStats(start, end);

        // 使用EasyExcel写入response流
        EasyExcel.write(response.getOutputStream(), DeviceStatsVO.class)
                .sheet("设备使用统计")
                .doWrite(data);
    }
}
